// =====================================================================================================================
// Configuration variables for Web services and authentication
// =====================================================================================================================

// CA Service Desk Web Service Credentials
var SERVICE_DESK_URL = "http://localhost:8080/axis/services/USD_R11_WebService";

var SERVICE_DESK_USER = "ServiceDeskAdmin";
var SERVICE_DESK_PASSWORD_FILE = "conf/caservicedesk.pwd";
var SERVICE_DESK_PASSWORD = getPassword(SERVICE_DESK_PASSWORD_FILE);

// Determines whether the data sync summary should be send to the xMatters Administrator
var SEND_SYNC_SUMMARY = true;
var DATA_LOAD_EXCEPTION_LOG_MESSAGE = "Data Load operation terminated with exception";

// The default user that quick message notifications will target for data sync reporting
var XMATTERS_ADMINISTRATOR = "companyadmin";

// =====================================================================================================================
// Configuration variables that determine which users and groups are transferred from CA SDM to xMatters, and
// what user/group information is retrieved.
// =====================================================================================================================

// Determines how many records are in each chunk of a list as users and groups are processed.
// 250 is the CA SDM Maximum value.
var CHUNK_SIZE = 250;
var MAX_ROWS = "100";

// ---------------------------------------------------------------------------------------------------------------------
// Definitions of the WHERE clauses used to retrieve users and groups from CA SDM using the CA Web services.
// These can use any of the attributes of the Contact object with atomic data types and SREL (single reference),
// but not lists. SREL attributes can include can use the dot notation.
//
// NOTE: The users or groups that are transferred from CA SDM to xMatters is also affected by the settings in
// dataSyncList.js
// ---------------------------------------------------------------------------------------------------------------------

// This where clause selects CA SDM contacts who correspond to xMatters users and should be transferred to xMatters.
var WHERE_CLAUSE_USERS = "( (type.sym = \'Analyst\') OR (type.sym = \'Help Desk\') "
    + "OR (type.sym = \'Manager\') OR (type.sym = \'Operator\') OR (type.sym = \'Technician\') ) "
    + " AND (delete_flag.sym = \'Active\')";

// This where clause selects CA SDM contacts that correspond to xMatters groups and should be transferred to xMatters.
var WHERE_CLAUSE_GROUPS = "(type.sym = \'Group\') AND (delete_flag.sym = \'Active\')";

// Used to determine the contacts that are members of a particular group AND who qualify for being transferred to xMatters.
//
// *****************************************************************************************************************
// * This MUST include an AND clause that duplicates the logic of WHERE_CLAUSE_USERS but uses the member attribute *
// * of the grpmem object.                                                                                         *
// *****************************************************************************************************************
var WHERE_CLAUSE_GROUP_MEMBERS = "(group = U'${GROUP_ID}') AND "
    + " ( (member.type.sym = \'Analyst\') OR (member.type.sym = \'Help Desk\') "
    + "OR (member.type.sym = \'Manager\') OR (member.type.sym = \'Operator\') OR (member.type.sym = \'Technician\') ) "
    + " AND (member.delete_flag.sym = \'Active\')";

// Used to determine the Contact roles for a given contact ID
var WHERE_CLAUSE_CONTACT_ROLES = "(contact = U'${CONTACT_ID}')";

// ---------------------------------------------------------------------------------------------------------------------
// Constants that determine which Object attributes are retrieved from CA SDM Web service operations
// ---------------------------------------------------------------------------------------------------------------------
var ATTRIBUTES_CNT_OBJECT = ["alt_phone", "type", "email_address", "fax_phone", "first_name", "delete_flag", "last_name", "beeper_phone", "pemail_address", "phone_number", "userid", "id"];
var ATTRIBUTES_GRPMEM_OBJECT = ["group.last_name", "member.userid"];
var ATTRIBUTES_CONTACT_ROLES = ["is_default", "contact.userid", "role_obj.name"];

// =====================================================================================================================
// Default values for properties that xMatters requires but CA SDM may not provide
// =====================================================================================================================
var DEFAULT_USER_FIRST_NAME = "Undefined";

// =====================================================================================================================
// If MAP_USER_ROLES is false, any newly created xMatters user is assigned the role(s) defined in DEFAULT_XMATTERS_ROLES,
// and the user roles are subsequently not changed if the user is updated. Multiple roles should be provided as a comma-
// separated list e.g. ["role1", "role2"]
//
// If MAP_USER_ROLES is true, xMatters roles are assigned to users based on their CA Service Desk roles and the value
// of the roleMap variable.
// =====================================================================================================================
var MAP_USER_ROLES = false;
var DEFAULT_XMATTERS_ROLES = ["Standard User"];

var roleMap = [];
roleMap["Service Desk Manager"] = ["Group Supervisor", "Person Supervisor"];

// Additional role mapping entries can be added, using the following as an example
// roleMap["CA SDM Role"] = ["xMatters Role 1", "xMatters Role 2"];

// =====================================================================================================================
// Configuration variables that determine properties of xMatters users that don't come from CA SDM
// =====================================================================================================================

// ---------------------------------------------------------------------------------------------------------------------
// Determines the site that newly created users will belong to
// ---------------------------------------------------------------------------------------------------------------------
var DEFAULT_USER_SITE = "Default Site";

// ---------------------------------------------------------------------------------------------------------------------
// Determines the default user service providers for newly created user devices in xMatters
// ---------------------------------------------------------------------------------------------------------------------
var DEFAULT_PROVIDER_EMAIL = "(x)matters Email Gateway";
var DEFAULT_PROVIDER_TEXT  = "(x)matters SMS Gateway";
var DEFAULT_PROVIDER_VOICE = "(x)matters Voice Gateway";

// ---------------------------------------------------------------------------------------------------------------------
// Constants used to set the isExternallyOwned properties of all xMatters users or groups
// ---------------------------------------------------------------------------------------------------------------------
var EXTERNALLY_OWN_USERS = false;
var EXTERNALLY_OWN_GROUPS = false;

// ---------------------------------------------------------------------------------------------------------------------
// The Web Login Type must be one of "NATIVE" or "LDAP". If set to "LDAP", then the LDAP domain must be valid
// ---------------------------------------------------------------------------------------------------------------------
var WEB_LOGIN_TYPE = "NATIVE";
var WEB_LOGIN_LDAP_DOMAIN = "company.com";

// ---------------------------------------------------------------------------------------------------------------------
// Default supervisor for users and groups. This will be companyadmin unless you have merged xMatters admin accounts,
// in which case it needs to be superadmin
// ---------------------------------------------------------------------------------------------------------------------
var DEFAULT_SUPERVISOR = ["companyadmin"];

// ---------------------------------------------------------------------------------------------------------------------
// Mappings between the international dialing prefix for a country and the ISO 3166-1 alpha-2 2-character country code.
// The ISO 3166-1 codes are needed when telephone devices are added to xMatters.
//
// If more than one country entry has the same dialing prefix (e.g. the US and Canada) the first match in the list
// will be used.
//
// For more ISO 3166-1 alpha-2 codes see http://en.wikipedia.org/wiki/ISO_3166-1_alpha-2
// ---------------------------------------------------------------------------------------------------------------------
var countryCodes =
    <countryCodes>
        <country>
            <dialingPrefix>1</dialingPrefix>
            <ISO_3166_1>US</ISO_3166_1>
        </country>
        <country>
            <dialingPrefix>1</dialingPrefix>
            <ISO_3166_1>CA</ISO_3166_1>
        </country>
    </countryCodes>;


// =====================================================================================================================
// Global constants
// =====================================================================================================================
var log = new Logger("CA Service Desk data load: ");

var OBJECT_TYPE_CONTACT = "cnt";
var OBJECT_TYPE_GROUP_MEMBER = "grpmem";
var OBJECT_TYPE_CONTACT_ROLES = "cnt_role";

var ACTION_LOAD = "load";

var PROCESS_ADD_UPDATE = false;

var ALLOW_ADD_USER = false;
var ALLOW_ADD_GROUP = false;

var DEFAULT_TEAM_NAME = "Roster";
var DEFAULT_TEAM_DESC = "This team is managed by the CA SDM data sync integration";
var DEFAULT_COVERAGE_NAME = "24/7"; 

var SYNC_SUMMARY_SUBJECT = "xMatters (CA Service Desk DataLoad) Report";
