importPackage(java.util);
importPackage(java.io);
importPackage(java.lang);
importClass(Packages.org.apache.commons.httpclient.HttpVersion);
importClass(Packages.com.xmatters.ca.servicedesk.util.ParseCAFile);
importClass(Packages.com.alarmpoint.integrationagent.apxml.APXMLMessage);
importClass(Packages.com.alarmpoint.integrationagent.apxml.APXMLToken);
importClass(Packages.com.thoughtworks.xstream.XStream);
importClass(Packages.com.thoughtworks.xstream.converters.reflection.PureJavaReflectionProvider);
importClass(Packages.com.alarmpoint.integrationagent.security.EncryptionUtils);
importClass(Packages.com.alarmpoint.integrationagent.util.EventDeduplicator);

var xStream = new XStream(new PureJavaReflectionProvider())

var IA_UTIL_VERSION = "lib_1_2_6";
var CASD_VERSION = "-1-6-1";

load("integrationservices/caservicedesk" + CASD_VERSION + "/lib/" + IA_UTIL_VERSION + "/javascript/core/baseclass.js");
load("integrationservices/caservicedesk" + CASD_VERSION + "/lib/" + IA_UTIL_VERSION + "/javascript/core/logger.js");
load("integrationservices/caservicedesk" + CASD_VERSION + "/lib/" + IA_UTIL_VERSION + "/javascript/core/util.js");
load("integrationservices/caservicedesk" + CASD_VERSION + "/lib/" + IA_UTIL_VERSION + "/javascript/webservices/wsutil.js");
load("integrationservices/caservicedesk" + CASD_VERSION + "/lib/" + IA_UTIL_VERSION + "/javascript/xmatters/xmattersws.js");
load("integrationservices/caservicedesk" + CASD_VERSION + "/lib/" + IA_UTIL_VERSION + "/javascript/xmatters/xMattersEventWS.js");

load("integrationservices/caservicedesk" + CASD_VERSION + "/caservicedesk" + CASD_VERSION + "/configuration.js");
load("integrationservices/caservicedesk" + CASD_VERSION + "/caservicedesk" + CASD_VERSION + "/caservicedeskws.js");
load("integrationservices/caservicedesk" + CASD_VERSION + "/caservicedesk" + CASD_VERSION + "/transitions.js");
load("integrationservices/caservicedesk" + CASD_VERSION + "/caservicedesk" + CASD_VERSION + "/json2.js");

var exceptionString = "JavaException: ";

var notePrefix = "[xMatters] - "; // The Prefix which is added to the note comments for AlarmPoint e.g. "[xMatters] " 

var presentationSubjectPrefix = "xMatters (CA SDM)"; // The Prefix to be added to notification subject in mobile xMatters app

var notificationSender = "CA ServiceDesk Manager";  // A name to appear in xMatters UI as a notification sender

var TOKEN_NAME = "apiaResult";
var TOKEN_NAME_EXCEPTION = "apiaResultException";

var TOKEN_NAME_FORMATTED_EXCEPTION = "apiaFormattedException";  // Optional pre-formatted text for notification that will be sent to the user 
var TOKEN_NAME_FORMATTED_LOG_ENTRY = "apiaFormattedLogEntry";   // Optional pre-formatted text for xMatters log entry 

var LOCKED_RECORD_INDICATOR = "<ErrorCode>1002</ErrorCode>"; // an indicator of ServiceDesk Manager's "Record is locked" exception 

var SERVICE_DESK_PASSWORD = getPassword(SERVICE_DESK_PASSWORD_FILE);

var caServiceDeskWS = new CAServiceDeskWS(SERVICE_DESK_URL);

// This is the main method for accepting JSON HTTP requests from CA Service Desk.
// The incoming HTTP JSON Object must formatted correctly.
function apia_http( httpRequestProperties, httpResponse )
{
    try 
    {
      log.debug("Received a JSON HTTP request from CA Service Desk with httpRequestProperties: " + httpRequestProperties);
      
      var httpRequest = httpRequestProperties.getProperty("http.request");  
      var requestBody = httpRequestProperties.getProperty("REQUEST_BODY");
      log.info("Received JSON HTTP request from CA Service Desk with request body: " + requestBody);
      var payload = JSON.parse(requestBody);

      createApxmlFromCaJson(payload);

      // send the response back to the client
      httpResponse.setStatusLine(HttpVersion.HTTP_1_1, 200);
      httpResponse.setBodyString("JSON data received for processing by xMatters Integration agent.");
      return httpResponse;
    } 
    catch (e)
    {
      log.error("*********** Processing JSON failed ***********")
      log.error("An error occurred while processing the incoming JSON data: " + e.toString());
      
      // send error response back to the client
      httpResponse.setStatusLine(HttpVersion.HTTP_1_1, 500);
      httpResponse.setBodyString("Error occurred processing the incoming JSON data: " + e.toString());
      return httpResponse;
    }
}

/**
 * This is the main method for handling APXML messages sent to the 
 * Integration Service by APClient.  APClient sends the location of the CA Service Desk notification file.
 * This notification file should be created as part of the xMatters notification method.  The file is then
 * parsed for parameters which will be sent to AlarmPoint as event tokens.
 * <p>
 * For example, the key NX_NTF_MESSAGE_HTML is found in the notification file and is sent to AlarmPoint
 * as NOTIFICATION_MESSAGE_HTML_0 , NOTIFICATION_MESSAGE_HTML_1.  It is broken up as each event token is
 * limited to 4000k.  Once recieved by AlarmPoint these tokens are re-assembled and sent as an html notification
 * to the targeted recipient.
 * <p>
 * Any APXMLMessage object that this method returns will be sent to AlarmPoint
 * via the Integration Service's outbound queues.
 */
function apia_input(apxml) 
{
  // Parse file
  createApxmlFromCaNotificationFile(apxml, apxml.getValue("notification_file"));
}

/**
 * Handle the notification from CA Service Desk
 * @param apxml notification content
 * @param notificationFile name of the notification file to be parsed
 * @return enriched APXML to send to xMatters
 */
function createApxmlFromCaNotificationFile(apxml, notificationFile)
{
  log.debug("Received new injection from CA Service Desk.  Parsing notification file: " + notificationFile);
  var fileParser = new ParseCAFile(notificationFile);
  var parsedFile = fileParser.parse();

  // Add object section to APXML
  var parsedFileKeys = (parsedFile.keySet()).toArray();
  var objectSectionKey;
  for(var i=0, len=parsedFileKeys.length; i<len; ++i)
  {
    if(parsedFileKeys[i] != "cnt" && parsedFileKeys[i] != "notification")
    {
      objectSectionKey = parsedFileKeys[i];
    }
  }
  var objectSection = parsedFile.get(objectSectionKey);

  if ("chg".equalsIgnoreCase(objectSectionKey))
  {
    apxml.setToken("NX_NTF_REF_NUM", objectSection.get("NX_NTF_CHG_REF_NUM"));
    log.debug("NX_NTF_REF_NUM: " + objectSection.get("NX_NTF_REF_NUM"));
  }
  else
  {
    apxml.setToken("NX_NTF_REF_NUM", objectSection.get("NX_NTF_REF_NUM"));
    log.debug("NX_NTF_REF_NUM: " + objectSection.get("NX_NTF_REF_NUM"));
  }
  apxml.setToken("NX_NTF_PERSISTENT_ID", objectSection.get("NX_NTF_PERSISTENT_ID"));
  apxml.setToken("incident_id", objectSection.get("NX_NTF_PERSISTENT_ID"));
  apxml.setToken(APXMLMessage.APIA_PROCESS_GROUP, objectSection.get("NX_NTF_PERSISTENT_ID"));
  apxml.setToken("NX_NTF_TYPE_SYM", objectSection.get("NX_NTF_TYPE.SYM"));
  apxml.setToken("NX_NTF_STATUS_SYM", objectSection.get("NX_NTF_STATUS.SYM"));
  apxml.setToken("NX_NTF_PRIORITY_SYM", objectSection.get("NX_NTF_PRIORITY.SYM"));
  apxml.setToken("NX_NTF_CUSTOMER_COMBO_NAME", objectSection.get("NX_NTF_CUSTOMER.COMBO_NAME"));
  apxml.setToken("NX_NTF_ASSIGNEE_COMBO_NAME", objectSection.get("NX_NTF_ASSIGNEE.COMBO_NAME"));
  apxml.setToken("NX_NTF_DESCRIPTION", objectSection.get("NX_NTF_DESCRIPTION"));
  apxml.setToken("TICKET_SUMMARY", objectSection.get("NX_NTF_SUMMARY"));
  
  // Add notification section to APXML
  var notificationSection = parsedFile.get("notification");
  apxml.setToken("NOTIFICATION_SUMMARY", notificationSection.get("NX_NTF_SUMMARY"));
  log.debug("Notification message is: " + notificationSection.get("NX_NTF_MESSAGE"));
  apxml.setToken("NOTIFICATION_MESSAGE", notificationSection.get("NX_NTF_MESSAGE"));

  // Add subject to support mobile app (see SCO-4959 for details)
  var subject = notificationSection.get("NX_NTF_SUMMARY");
  
  if (presentationSubjectPrefix.trim().length > 0) {
    subject = presentationSubjectPrefix + ": " + subject;
  }

  log.debug("Notification subject: " + subject); 
  apxml.setToken("SUBJECT", subject );
  
  log.debug("Notification sender: " + notificationSender); 
  apxml.setToken("SENDER", notificationSender );
	
  var notification_userid = notificationSection.get("NX_NTF_USERID");  
  log.debug("Notification will be sent to userID: " + notification_userid + "; user name is: " + notificationSection.get("NX_NTF_COMBO_NAME"));
  apxml.setToken("NOTIFICATION_USERID", notification_userid);
  apxml.setToken("NOTIFICATION_COMBO_NAME", notificationSection.get("NX_NTF_COMBO_NAME"));
  var nx_ntf_urgency = notificationSection.get("NX_NTF_URGENCY");
  // nx_ntf_urgency is represented in the CA SDM notification file as an integer.
  // There is a set of key value pairs in the notification file which seem to indicate that
  // 1 maps to a low notification and 4 maps to an emergency notification.
  // This association is the assumption made to create the if block below.
  // Symbols from notification file:
  // NX_NTF_NOTIFY_URGENCY1.SYM=Low
  // NX_NTF_NOTIFY_URGENCY2.SYM=Normal
  // NX_NTF_NOTIFY_URGENCY3.SYM=High
  // NX_NTF_NOTIFY_URGENCY4.SYM=Emergency
  if(nx_ntf_urgency > 2)
  {
    apxml.setToken("apia_priority", "high");
    log.debug("APIA_PRIORITY: " + notificationSection.get("NX_NTF_URGENCY"));
  }
    
  // Add cnt section to APXML
  var cntSection = parsedFile.get("cnt");
  apxml.setToken("CNT_PERSISTENT_ID", cntSection.get("NX_NTF_PERSISTENT_ID"));
  // log.debug("CNT_PERSISTENT_ID: " + cntSection.get("NX_NTF_PERSISTENT_ID"));
  apxml.setToken("CNT_NOTIFY_URGENCY_SYM", cntSection.get("NX_NTF_NOTIFY_URGENCY" + nx_ntf_urgency + ".SYM"));
  // log.debug("CNT_NOTIFY_URGENCY_SYM: " + cntSection.get("NX_NTF_NOTIFY_URGENCY" + nx_ntf_urgency + ".SYM"));

  // Event information
  // log.debug("REQUESTOR_NAME: " + objectSection.get("NX_NTF_REQUESTED_BY.COMBO_NAME"));
  apxml.setToken("REQUESTOR_NAME", objectSection.get("NX_NTF_REQUESTED_BY.COMBO_NAME")); 
  // log.debug("ASSIGNEE_NAME: " + objectSection.get("NX_NTF_ASSIGNEE.COMBO_NAME"));
  apxml.setToken("ASSIGNEE_NAME", objectSection.get("NX_NTF_ASSIGNEE.COMBO_NAME"));
  
  // log.debug("PRIORITY: " + objectSection.get("NX_NTF_PRIORITY.SYM"));
  apxml.setToken("PRIORITY", objectSection.get("NX_NTF_PRIORITY.SYM"));
  // log.debug("STATUS: " + objectSection.get("NX_NTF_STATUS.SYM"));
  apxml.setToken("STATUS", objectSection.get("NX_NTF_STATUS.SYM"));
  // log.debug("URGENCY: " + objectSection.get("NX_NTF_URGENCY.SYM"));
  apxml.setToken("URGENCY", objectSection.get("NX_NTF_URGENCY.SYM"));
  // log.debug("IMPACT: " + objectSection.get("NX_NTF_IMPACT.SYM"));
  apxml.setToken("IMPACT", objectSection.get("NX_NTF_IMPACT.SYM"));
  // log.debug("SEVERITY: " + objectSection.get("NX_NTF_SEVERITY.SYM"));
  apxml.setToken("SEVERITY", objectSection.get("NX_NTF_SEVERITY.SYM"));

  if(notification_userid != null && notification_userid != "")
  {
  	// log.debug("Setting recipients to value of NX_NTF_USERID: " + notification_userid);
  	apxml.setToken("recipients", notification_userid);
  }
  else
  {
    // log.debug("Setting recipients to value of NX_NTF_COMBO_NAME: " + notificationSection.get("NX_NTF_COMBO_NAME"));
    apxml.setToken("recipients", notificationSection.get("NX_NTF_COMBO_NAME"));
  }
  
  // Break up the HTML String into 4000 char chunks so that it can be processed by the APS.
  var htmlString = notificationSection.get("NX_NTF_MESSAGE_HTML");
  if (htmlString != null)
  {
    // Java String
    // log.debug("HTML String length: " + htmlString.length());
    apxml.setToken("CA_HTML_NOTIFICATION_LENGTH", htmlString.length());
    htmlString = htmlString.replaceAll("[\\r\\n]", "");
    var chunkedString = htmlString.match(RegExp('.{1,4000}', 'g'));

    // JavaScript String
    // log.debug("Size of chunk array: " + chunkedString.length);
    apxml.setToken("CA_HTML_NOTIFICATION_CHUNKS", chunkedString.length);
    for(var i=0, len=chunkedString.length; i<len; ++i)
    {
      // log.debug("HTML chunk "+ i + ": " + chunkedString[i]);
      apxml.setToken("XM_NOTIFICATION_MESSAGE_HTML_"+i, chunkedString[i]);
    }
  }
  
  // Add the valid response choices to the APXML
  var ticketFactory = objectSectionKey;
  if ("cr".equalsIgnoreCase(ticketFactory))
  {
    // This is Problem/Incident/Request...retrieve the ticket factory information for the transitions
    ticketFactory = caServiceDeskWS.getTicketFactory(objectSection.get("NX_NTF_TYPE.SYM"))
  }

  var responseList
  if (USE_DYNAMIC_TRANSITIONS)
  {
    responseList = caServiceDeskWS.getResponseOptions(objectSection.get("NX_NTF_PERSISTENT_ID"), ticketFactory);
  }
  else
  {
    responseList = getStaticResponseOptions(ticketFactory, objectSection.get("NX_NTF_STATUS.SYM"));
  }
  apxml.setToken("TICKET_FACTORY", ticketFactory);
  apxml.setToken("RESPONSE_OPTIONS", responseList.toString());
  apxml.setToken("RESPONSE_OPTIONS-list-item-delimiter", ",");
  apxml.setToken("JSON", "false");

  // Send enriched APXML 
  submitAPXML( apxml );

  // markProcessed() will attempt to move the parsed notification file to a subdirectory called xmatters.
  // This is bound by file system access so if you don't want this feature just comment out fileParser.markProcessed().
  fileParser.setCopyAfterProcessing(false);
  fileParser.markProcessed();
  //fileParser.markProcessed('<custom directory>'); // If you want a different directory, use full qualified path.
  //fileParser.markProcessed('<custom directory>', '<custom filename>'); // If you want to control directory and file name.
}

/**
 * Forward enriched APXML to AlarmPoint, subject to deduplication
 * @param apxml APXML to be sent
 */
function submitAPXML(apxml)
{
  var webService = new xMattersWS();
  log.debug("Received an injection with tokens: " + webService.convertTokensToString(apxml));

  try
  {
    if (webService.submitAPXML(apxml, DELETE_EXISTING_EVENTS))
    {
        // If submitAPXML() actually sent the message and didn't deduplicate it, add an annotation to the IA logs.
        log.info("Notification will be sent to [" + apxml.getValue("recipients") + "].");
    }
  }
  catch (ex)
  {
    log.error("an exception was encountered while sending APXML. Exception is: " + ex.toString());
    throw ex;
  } 
}

/**
 * Handle the notification from CA Service Desk
 * This function takes in a JSON Object and creates a APXML message from it.
 * The message is then sent to xMatters via the web services IAUtils calls.
 * @param payload, the JSON Object from CA SD
 */
function createApxmlFromCaJson(payload)
{
  log.debug("Entering createApxmlFromCaJson()...");
  var apxml = ServiceAPI.createAPXML();
  apxml.setMethod("ADD");
  apxml.setSubclass("action"); 
  
  // get the 3 sections of the JSON Message
  // Expecting to have either obj + act + ntf or obj + ntf
  var sdObjectMain;
  var sdObjectAct;
  var sdObjectNtf;
  
  for ( var i=0; i < payload.Notification.SDObject.length; i++ ) 
  {
    if (payload.Notification.SDObject[i]["@attributes"].section.equals("cr"))  
	  sdObjectMain = payload.Notification.SDObject[i];
    if (payload.Notification.SDObject[i]["@attributes"].section.equals("act")) 
	{
	  sdObjectAct = payload.Notification.SDObject[i];
	  apxml.setToken("ACT_TYPE", sdObjectAct.type);
      apxml.setToken("ACT_DESCRIPTION", decodeStringWithNLCR(sdObjectAct.description));
      apxml.setToken("ACT_ACTION_DESC", decodeStringWithNLCR(sdObjectAct.action_desc));  
	}
    if (payload.Notification.SDObject[i]["@attributes"].section.equals("ntf"))  
	  sdObjectNtf = payload.Notification.SDObject[i];
  }
 
  if (sdObjectMain == null || sdObjectNtf == null ) 
  {
    throw new Exception("JSON Message does not have a ACT or NTF section");
  }
  
  if (false) //"chg".equalsIgnoreCase(objectSectionKey)) // Don't need this now.
  {
    apxml.setToken("NX_NTF_REF_NUM", objectSection.get("NX_NTF_CHG_REF_NUM"));
  }
  else
  {
    // Grab the ref_num as an example
    log.debug("payload: " + payload);
    //log.debug("NX_NTF_REF_NUM: " + sdObjectMain.ref_num);
    apxml.setToken("NX_NTF_REF_NUM", sdObjectMain.ref_num);
  }

  apxml.setToken("NX_NTF_PERSISTENT_ID", sdObjectMain.persistent_id);
  apxml.setToken("incident_id", sdObjectMain.persistent_id);
  apxml.setToken(APXMLMessage.APIA_PROCESS_GROUP, sdObjectMain.persistent_id);
  apxml.setToken("NX_NTF_TYPE_SYM", sdObjectMain["type.sym"]);
  apxml.setToken("NX_NTF_STATUS_SYM", sdObjectMain["status.sym"]);
  apxml.setToken("NX_NTF_PRIORITY_SYM", sdObjectMain["priority.sym"]);
  apxml.setToken("NX_NTF_CUSTOMER_COMBO_NAME", sdObjectMain["customer.combo_name"]);
  apxml.setToken("NX_NTF_ASSIGNEE_COMBO_NAME", sdObjectMain["assignee.combo_name"]);
  apxml.setToken("NX_NTF_DESCRIPTION", decodeStringWithNLCR(sdObjectMain.description));
  apxml.setToken("TICKET_SUMMARY", decodeStringWithNLCR(sdObjectMain.summary));

  // Add notification section to APXML
  apxml.setToken("NOTIFICATION_SUMMARY", decodeStringWithNLCR(sdObjectNtf.summary));
  apxml.setToken("NOTIFICATION_MESSAGE", decodeStringWithNLCR(sdObjectNtf.message));

  var notification_userid = sdObjectMain["assignee.userid"];  
  var notification_combo_name = sdObjectMain["assignee.combo_name"];  
  log.info("Notification with incidentID " + sdObjectMain.persistent_id + " will be sent to userID: " + notification_userid + "; user name is: " + notification_combo_name);
  apxml.setToken("NOTIFICATION_USERID", notification_userid);
  apxml.setToken("NOTIFICATION_COMBO_NAME", notification_combo_name);
  var nx_ntf_urgency = sdObjectMain.urgency;
  // nx_ntf_urgency is represented in the CA SDM notification file as an integer.
  // There is a set of key value pairs in the notification file which seem to indicate that
  // 1 maps to a low notification and 4 maps to an emergency notification.
  // This association is the assumption made to create the if block below.
  // Symbols from notification file:
  // NX_NTF_NOTIFY_URGENCY1.SYM=Low
  // NX_NTF_NOTIFY_URGENCY2.SYM=Normal
  // NX_NTF_NOTIFY_URGENCY3.SYM=High
  // NX_NTF_NOTIFY_URGENCY4.SYM=Emergency
  if(nx_ntf_urgency > 2)
  {
    apxml.setToken("apia_priority", "high");
    log.debug("APIA_PRIORITY: " + sdObjectMain.urgency);
  }
  
  apxml.setToken("CNT_NOTIFY_URGENCY_SYM", sdObjectNtf.urgency);
  
  // Subscription information
  apxml.setToken("REQUESTOR_NAME", sdObjectMain["requested_by.combo_name"]);
  apxml.setToken("ASSIGNEE_NAME", sdObjectMain["assignee.combo_name"]);
  apxml.setToken("PRIORITY", sdObjectMain["priority.sym"]);
  apxml.setToken("STATUS", sdObjectMain["status.sym"]);
  apxml.setToken("URGENCY", sdObjectMain["urgency.sym"]);
  apxml.setToken("IMPACT", sdObjectMain["impact.sym"]);
  apxml.setToken("SEVERITY", sdObjectMain["severity.sym"]);

  if(notification_userid != null && notification_userid != "")
  {
  	// log.debug("Setting recipients to value of NX_NTF_USERID: " + notification_userid);
  	apxml.setToken("recipients", notification_userid);
  }
  else
  {
    var recipients = "";
    for(var i=0; i<sdObjectNtf.contact.length; i++)
    {
      recipients += sdObjectNtf.contact[i].ntf_userid + ","; 
    }
    
    apxml.setToken("recipients", recipients);
  }

  // Break up the HTML String into 4000 char chunks so that it can be processed by the APS.
  var htmlString = new java.lang.String(sdObjectNtf.message_html);

  if (htmlString != null && htmlString != "")
  {
    // Java String
    //log.debug("HTML String length: " + htmlString.length());
    apxml.setToken("CA_HTML_NOTIFICATION_LENGTH", htmlString.length());
    htmlString = htmlString.replaceAll("[\\r\\n]", "");
    var chunkedString = htmlString.match(RegExp('.{1,4000}', 'g'));

    // JavaScript String
    //log.debug("Size of chunk array: " + chunkedString.length);
    apxml.setToken("CA_HTML_NOTIFICATION_CHUNKS", chunkedString.length);
    for(var i=0, len=chunkedString.length; i<len; ++i)
    {
      // log.debug("HTML chunk "+ i + ": " + chunkedString[i]);
      apxml.setToken("XM_NOTIFICATION_MESSAGE_HTML_"+i, chunkedString[i]);
    }
  }
  
  // Add the valid response choices to the APXML
  var ticketFactory = sdObjectMain["@attributes"].section;
  if ("cr".equalsIgnoreCase(ticketFactory))
  {
    // This is Problem/Incident/Request...retrieve the ticket factory information for the transitions
    ticketFactory = caServiceDeskWS.getTicketFactory(sdObjectMain["type.sym"]);
  }

  var responseList
  if (USE_DYNAMIC_TRANSITIONS)
  {
    responseList = caServiceDeskWS.getResponseOptions(sdObjectMain.persistent_id, ticketFactory);
  }
  else
  {
    responseList = getStaticResponseOptions(ticketFactory, sdObjectMain["status.sym"]);
  }
  apxml.setToken("TICKET_FACTORY", ticketFactory);
  apxml.setToken("RESPONSE_OPTIONS", responseList.toString());
  apxml.setToken("RESPONSE_OPTIONS-list-item-delimiter", ",");

  apxml.setToken("JSON", "true");

  log.debug("Apxml was successfully created.  Sending to xMatters...");

  // Send enriched APXML
  submitAPXML( apxml );
}

/**
 * CA will be encoding their strings to have the \n and \r characters replaced with __NL and __CR.
 * We need to decode each string and replace the characters.
 * @param string to be decoded
 * @return string which has the proper \n and \r characters
 */
function decodeStringWithNLCR(string) 
{
  if (string != "" && string != null) {
    return string.replace(/__NL/g, "\n").replace(/__CR/g, "\r");
  }
  return "";
}

/**
 * The main routine for handling del injections from the management system to AlarmPoint
 * As the CA SDM notifications are not based on event, a DEL event will be sent ahead of
 * every add event to terminate any existing events for the ticket.
 * @param incidentId id of the incident event that will be terminated
 */  
function handleDel(incidentId, priority)
{
  var apxml = ServiceAPI.createAPXML();
  apxml.setMethod("DEL");
  apxml.setSubclass("action");
  apxml.setToken("apia_priority", priority);
  apxml.setToken(APXMLMessage.APIA_PROCESS_GROUP, incidentId);
  apxml.setToken("incident_id", incidentId);

  log.debug("handleDel apxml: " + apxml);
  
  var response = initializeResponse(apxml);
  try
  {
	ServiceAPI.sendAPXML(apxml);      
	response.setToken(TOKEN_NAME, "OK", APXMLToken.Type.STRING);
	log.debug("Finished sending apxml to AlarmPoint");
  }
  catch (e)
  {
	log.error("Caught exception processing [DEL]. Exception:" + e);
	response.setToken(TOKEN_NAME_EXCEPTION, e.toString().replace(exceptionString, ""), APXMLToken.Type.STRING);
  }

  log.debug(response);
}

// Not used in this integration, but the 5.0 IA wants this function to exist
function apia_webservice_init() {}

/**
 * This is the main method for handling APXML messages sent to the
 * Integration Service by AlarmPoint.  The messages may be responses to 
 * previous submissions, or they may be requests to the Integration Service
 * that originate from AlarmPoint 
 * (e.g., via an ExternalServiceMessage/ExternalServiceRequest2).
 * <p>
 * Any APXMLMessage object that this method returns will be sent to AlarmPoint
 * either via the Integration Service's outbound queues or directly as a 
 * Web Service response, depending on the mechanism used to submit the APXML.
 */
function apia_response(apxml) 
{
  var method = apxml.getMethod();
  
  if (APXMLMessage.OK_METHOD.equalsIgnoreCase(method))
  {
    return handleOK(apxml);
  } 
  else if (APXMLMessage.ERROR_METHOD.equalsIgnoreCase(method))
  {
    return handleERROR(apxml);
  } 
  else if (APXMLMessage.SEND_METHOD.equalsIgnoreCase(method))
  {
    return handleSend(apxml);
  }
  else if (APXMLMessage.REQUEST_METHOD.equalsIgnoreCase(method))
  {
    return handleRequest(apxml);
  }
  else
  {
    throw new IllegalArgumentException("Unrecognized APXML method: " + method);
  }
}

/**
 * OK responses are returned by AlarmPoint for each APXML message that it accepts
 * from an Integration Service.
 * <p>
 * Log the acknowledgement using the ping Integration Service's logger.
 */
function handleOK(apxml)
{
  var id = apxml.getTransactionId();
  ServiceAPI.getLogger().info("Received OK response from xMatters for transactionId " + id + ".");
}

/**
 * ERROR responses are returned by AlarmPoint for each APXML message that it
 * is unable to accept from an Integration Service.
 * <p>
 * Log the error using the ping Integration Service's logger.
 */
function handleERROR(apxml) 
{
  var id = apxml.getTransactionId();
  var code = apxml.getValue(APXMLMessage.ERROR_CODE); 
  ServiceAPI.getLogger().error("Received ERROR response from xMatter for transactionId " + id + " with error code " + code);
}

/**
 * The main routine for handling ExternalServiceMessages from AlarmPoint
 */
function handleSend(apxml)
{
  log.debug( "handleSend received apxml: " + apxml + " from xMatters.");

  // handle annotations coming from callout APS script 
  // they are formatted differently 
  // the presence of the message token means this message 
  // is a callout annotation.
  var message = apxml.getValue("message");
  var incidentId = apxml.getToken("incident_id").getValue();
  
  if ( message != null && incidentId != null)
  {
    handleCalloutAnnotate( apxml );
  }
  else
  {
    handleApsResponses( apxml );
  }
}

/**
 * The main routine for handling ExternalServiceRequests from AlarmPoint
 */
function handleRequest(apxml)
{
  log.debug( "handleRequest received apxml: " + apxml  + " via External Service Request from xMatters.");

  // handle annotations coming from callout APS script 
  // they are formatted differently 
  // the presence of the message token means this message 
  // is a callout annotation.
  var message = apxml.getValue("message");
  var incidentId = apxml.getToken("incident_id").getValue();

  if ( message != null && incidentId != null)
  {
    return handleCalloutAnnotate( apxml );
  }
  else
  {
    var response = initializeResponse(apxml);

    try
    {
      handleApsResponses( apxml );
      response.setToken(TOKEN_NAME, "OK", APXMLToken.Type.STRING);
    }
    catch (e)
    {
      var errMsg = "Caught exception while processing response from xMatters: " + e;
       
      // INTA-2947: CA SDM - Handle locked records when processing message responses
      if ( errMsg.indexOf(LOCKED_RECORD_INDICATOR) > 0) 
      {
        // deserialize the additional tokens
        var additionalTokens = getAdditionalTokens(apxml);
        log.debug( "\tAPXML includes additionalTokens keys: " + additionalTokens);
        var incidentId = processAdditionalTokens(additionalTokens, "incident_id", false);
        var requestDescription = processAdditionalTokens(additionalTokens, "request_desc", false);

        log.error("\tEncountered an error processing response '" + requestDescription + "'. Record " + incidentId + " is locked. ");
 
        var errorMsgInd = "<ErrorMessage>";
        var errorMsgEnd = "</ErrorMessage>"; 
       
        var casdmError = errMsg.substring( errMsg.indexOf(errorMsgInd) + errorMsgInd.length, errMsg.indexOf(errorMsgEnd) );

        var xLogEntry = "Error processing response '" + requestDescription + "' as the CA SDM record " + incidentId + " is locked.  Error received from CA Service Desk: " + casdmError;
        var exceptionMessage = "Your response '" + requestDescription + "' was not processed as the CA SDM record " + incidentId +
           " is locked. To record your response, log into CA Service Desk and update the incident manually. \n\nCA Service Desk returned the following error message:\n\n" + casdmError;
      
        response.setToken(TOKEN_NAME_FORMATTED_LOG_ENTRY, xLogEntry, APXMLToken.Type.STRING);
        response.setToken(TOKEN_NAME_FORMATTED_EXCEPTION, exceptionMessage, APXMLToken.Type.STRING);      
      }
      else
      {
        log.error(errMsg);
        response.setToken(TOKEN_NAME_EXCEPTION, errMsg.replace(exceptionString, ""), APXMLToken.Type.STRING);
      }
    }
    log.debug("The following Response will be returned to xMatters: " + response);
    return response;
  }
}

/**
 * Used to update CA Service Desk Incidents using web services
 * <p>
 * The main routine for handling user responses to xMatters notifications.
 * <p>
 * Throw an exception that will be logged by the Integration Agent.
 */
function handleApsResponses(apxml) 
{
  var incidentId = apxml.getToken("incident_id").getValue();
  log.debug( "About to handle user response from xMatters for incidentId: " + incidentId);


  var requestText = "";
  var messageText = notePrefix + "Updated Task";
  if(apxml.getToken("request_text") != null)
  {
    requestText = apxml.getToken("request_text").getValue();
    log.debug("\tThe response includes request text: " + requestText);
  }
  if(apxml.getToken("message_text") != null && !"".equals(apxml.getToken("message_text")))
  {
    messageText = notePrefix + apxml.getToken("message_text").getValue();
    log.debug("\tThe response includes message text: " + messageText);
  }

  // deserialize the additional tokens
  var additionalTokens = getAdditionalTokens(apxml);
  log.debug( "\tThe response contains additionalTokens keys: " + additionalTokens);

  var objectHandle = processAdditionalTokens(additionalTokens, "object_handle", false);     
  var target = processAdditionalTokens(additionalTokens, "target", false);
  var assignee = processAdditionalTokens(additionalTokens, "assignee", false);       
  var response = processAdditionalTokens(additionalTokens, "response", false);     

  if (objectHandle != null && !"".equals(objectHandle))
  {
      var user;  
      if ( ENABLE_IMPERSONATE )
      {
        user = null;
        
        if (response == null) 
        {
          log.debug("not a response from a user or a device. User '" + ANNOTATION_USER + "' will be used for annotation");
          
          try
          {
            user = caServiceDeskWS.findContacts(ANNOTATION_USER);
          }
          catch (e)
          {
            // this is not a critical error. print a warning and continue
            log.warn("Caught exception processing response[" + requestText + "] for [" + incidentId + "]. Exception:" + e);
          }
          
          if (user == null) 
          {
            log.warn("User '" + ANNOTATION_USER + "' was not found. annotations will use '" + (target != null ? target : SERVICE_DESK_USER) + "' user");
          }
          else 
          { // remove note prefix from log entries made by 'xMatters' user
            messageText = messageText.replace(notePrefix, "");
          }   
        } 
        else 
        {
          log.debug("The response from a user or a device will use " + (target != null ? target : SERVICE_DESK_USER) + " for annotation");
        }

        if (user == null)
        {
          user = caServiceDeskWS.findContacts((target != null ? target : SERVICE_DESK_USER));
        }
      }
      else
      {
        user = caServiceDeskWS.findContacts((SERVICE_DESK_USER));
      }

      if ( !requestText.equalsIgnoreCase( "Annotate" ) && !requestText.equalsIgnoreCase( "IGN" ) )
      {
        caServiceDeskWS.changeStatus(user.handle, objectHandle, messageText, requestText);
        // Only set the assignee if the responder is the new assignee value.
        if (target.equals(assignee))
        {
          caServiceDeskWS.transfer(objectHandle, messageText, user.handle);
        }  
      }
      else
      {
        try 
        {
          caServiceDeskWS.annotate(objectHandle, user.handle, messageText);
        } 
        catch (e)
        {
          // INTA-2947: CA SDM - Handle locked records when processing message responses
          // Suppress locked record exceptions for responses that do not change the record
          if (e.toString().indexOf(LOCKED_RECORD_INDICATOR) < 0) 
          {
            throw e;
          }
        }
      }
  }
  
  log.debug("Finished call to management system for requestText: [" + requestText + "]");
}

/**
 * handles annotations sent from the Callout APS script
 */
function handleCalloutAnnotate( apxml )
{
  var messageText = apxml.getValue("message");

  var additionalTokens = getAdditionalTokens(apxml);
  log.debug( "additionalTokens keys: " + additionalTokens);
  var incidentId = processAdditionalTokens(additionalTokens, "incident_id", true); 
  var objectHandle = processAdditionalTokens(additionalTokens, "object_handle", true); 
    
  log.debug( "About to call to management system for callout annotate for [" + incidentId + "]" );

  try 
  {
    caServiceDeskWS.annotate(objectHandle, caServiceDeskWS.getHandleForUserId(SERVICE_DESK_USER), messageText);
  }
  catch (e)
  {
    log.error("Caught exception processing annotation for [" + incidentId + "] Exception:" + e);
  }
  
  log.debug("Finished call to management system for callout annotate");
}


/**
 * Retrieves the additional tokens from the APXML
 * @param apxml XML message to process
 * @return additional tokens
 * @throws if the additional tokens are missing from the XML.
 */
function getAdditionalTokens(apxml)
{
  if (apxml.getValue("additionalTokens") != null && !"".equalsIgnoreCase(apxml.getValue("additionalTokens"))) 
  {
    return xStream.fromXML(apxml.getValue("additionalTokens")); // LinkedHashMap
  }
  throw new Exception("additionalTokens is missing from apxml"); 
}

/**
 * Retrieves the additional tokens from the APXML and throws an exception if a mandatory token is missing
 * @param additionalTokens collection of tokens to process
 * @param tokenName name of the token to retrieve
 * @param mandatory true if token is required, false otherwise
 * @return the token value
 */
function processAdditionalTokens(additionalTokens, tokenName, mandatory)
{
	var token = additionalTokens.get(tokenName);
	if (mandatory && token == null)
	{
		throw new Exception("additionalTokens in apxml is missing " + tokenName + " which is mandatory");
	}
	return token;
}

/**
 * Initializes the response to be returned to the device
 * @param apxml APXML containing the request information
 * @return initialized instance of the response APXML
 */
function initializeResponse(apxml)
{
  var response = ServiceAPI.createAPXML();
  response.setMethod(APXMLMessage.RESPONSE_METHOD);
  response.setSubclass(APXMLMessage.RESPONSE_SUBCLASS);
  
  if (apxml.hasToken(APXMLMessage.REQUEST_ID))
  {
    response.setToken(APXMLMessage.REQUEST_ID, apxml.getValue(APXMLMessage.REQUEST_ID), APXMLToken.Type.STRING);
  }
  response.setToken(APXMLMessage.DESTINATION, apxml.getValue(APXMLMessage.ORIGINATOR), APXMLToken.Type.STRING);
  if (apxml.hasToken(APXMLMessage.APIA_PRIORITY)) 
  {
    response.setToken(APXMLMessage.APIA_PRIORITY, apxml.getValue(APXMLMessage.APIA_PRIORITY));
  }
  else
  {
    response.setToken(APXMLMessage.APIA_PRIORITY, APXMLMessage.APIA_PRIORITY_HIGH);
  }
  return response;
}

/**
 * Decrypts the password for the ServiceDeskUser used by the integration
 * @param passwordFile file/path of the file containing the encrypted password
 * @return decrypted password or an empty string if the password cannot be decrypted
 */
function getPassword(passwordFile)
{
  try
  {
    var encryptionUtils = new EncryptionUtils();
    var file = new File(passwordFile);
    return encryptionUtils.decrypt(file);
  }
  catch (e)
  {
    return "";
  }
}

/** 
 * Convert all tokens for this message into a parenthesis-grouped set of name-value-pairs. 
 * @param apxml tokens to be converted into a string
 * @return string representation of input tokens 
 */ 
function convertTokensToString(apxml) 
{ 
  var string = ""; 
  var tokenIterator = apxml.getTokens(); 
  while (tokenIterator.hasNext()) 
  { 
    var token = tokenIterator.next(); 
    string += "(" + token.getKey() + ", " + token.getValue() + ")"; 
  } 
  return string; 
}
